/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.basic;

import java.util.Vector;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.brain.Brain;
import re.jpayet.mentdb.core.session.Session;
import re.jpayet.mentdb.server.Start;

//The basic result JSON object
public class BasicResult {
	
	//Properties
	public JSONObject resultNode = null;
	
	//Constructor
	@SuppressWarnings("unchecked")
	public BasicResult(String userSession, String user, int type, String value, long seconds) {
		
		//Initialization
		resultNode = new JSONObject();
		
		//Add fields
		resultNode.put("user", user);
		resultNode.put("type", type);
		resultNode.put("msg", value);
		resultNode.put("s", seconds);
		resultNode.put("s", seconds);
		try {
			resultNode.put("mqlQueryMode", Session.allSessions.get(userSession).mqlQueryMode);
		} catch (Exception e) {resultNode.put("mqlQueryMode", false);};
		resultNode.put("ai", Start.AI_FIRST_NAME);
		if (Brain.listening) {
			resultNode.put("b", "RUNNING...");
		} else {
			resultNode.put("b", "STOPPED");
		}
		
	}
	
	//Constructor
	@SuppressWarnings("unchecked")
	public BasicResult(String userSession, String user, int type, String value, String[] currentUsers, Vector<String> target, Vector<String> userWhoTalkingWith, long seconds) {
		
		//Initialization
		resultNode = new JSONObject();
		
		//Add fields
		resultNode.put("user", user);
		resultNode.put("type", type);
		resultNode.put("msg", value);
		resultNode.put("s", seconds);
		resultNode.put("api-mql", Session.allSessions.get(userSession).isGrantedApiMql);
		resultNode.put("api-ai", Session.allSessions.get(userSession).isGrantedApiAi);
		try {
			resultNode.put("mqlQueryMode", Session.allSessions.get(userSession).mqlQueryMode);
		} catch (Exception e) {resultNode.put("mqlQueryMode", false);};
		resultNode.put("ai", Start.AI_FIRST_NAME);
		
		JSONArray array = new JSONArray();
		for(int i = 0;i<currentUsers.length;i++) {
			array.add(currentUsers[i]);
		}
		
		resultNode.put("currentUsers", array);
		
		JSONArray t = new JSONArray();
		for(int i = 0;i<target.size();i++) {
			t.add(target.get(i));
		}
		
		resultNode.put("target", t);
		
		JSONArray u = new JSONArray();
		u.add("ai");
		for(int i = 0;i<userWhoTalkingWith.size();i++) {
			u.add(userWhoTalkingWith.get(i));
		}
		
		resultNode.put("userWhoTalkingWith", u);

		if (Brain.listening) {
			resultNode.put("b", "RUNNING...");
		} else {
			resultNode.put("b", "STOPPED");
		}
		
		
	}
	
	/**
	 * @name toString
	 * @description Get the JSON string
	 */
	public String toString() {
		
		return resultNode.toJSONString();
		
	}

}