/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.link;

import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.basic.BasicData;
import re.jpayet.mentdb.core.db.Record;

public class LinkManager {
	
	//Create a new link
	@SuppressWarnings("unchecked")
	public static String create_thought(String user, String lang, String key, String value) throws Exception {
		
		//Generate an error if the link already exist
		if (exist_thought(lang, key)) {
			
			throw new Exception("Sorry, the link @[th_"+lang+"_"+key+"] already exist.");
			
		}
		
		BasicData bd = new BasicData();
		bd.load("{}");
		bd.dataNode.put("k", "@[th_"+lang+"_"+key+"]");
		bd.dataNode.put("v", value);

		Record.add(user, "@[th_"+lang+"_"+key+"]", bd.dataNode.toJSONString());
		
		return "@[th_"+lang+"_"+key+"]";

	}
	
	//Create a new link
	@SuppressWarnings("unchecked")
	public static String create_relation(String user, String lang, String key, String value) throws Exception {
		
		//Generate an error if the link already exist
		if (exist_relation(lang, key)) {
			
			throw new Exception("Sorry, the link @[r_"+lang+"_"+key+"] already exist.");
			
		}
		
		BasicData bd = new BasicData();
		bd.load("{}");
		bd.dataNode.put("k", "@[r_"+lang+"_"+key+"]");
		bd.dataNode.put("v", value);

		Record.add(user, "@[r_"+lang+"_"+key+"]", bd.dataNode.toJSONString());
		
		return "@[r_"+lang+"_"+key+"]";

	}
	
	//Delete a new link
	public static void delete_thought(String user, String lang, String key) throws Exception {
		
		//Generate an error if the link already exist
		if (!exist_thought(lang, key)) {
			
			throw new Exception("Sorry, the link @[th_"+lang+"_"+key+"] does not exist.");
			
		}
		
		Record.remove(user, "@[th_"+lang+"_"+key+"]");

	}
	
	//Delete a new link
	public static void delete_relation(String user, String lang, String key) throws Exception {
		
		//Generate an error if the link already exist
		if (!exist_relation(lang, key)) {
			
			throw new Exception("Sorry, the link @[r_"+lang+"_"+key+"] does not exist.");
			
		}
		
		Record.remove(user, "@[r_"+lang+"_"+key+"]");

	}
	
	//Get a link
	public static String get_value(String key) throws Exception {

		//Get the node
		JSONObject recNode = Record.getNode(key);
		
		//Generate an error if the link already exist
		if (recNode==null) {
			
			throw new Exception("Sorry, the link "+key+" does not exist.");
			
		}
		
		return recNode.get("v")+"";

	}
	
	//Get a link
	public static String get_thought(String lang, String key) throws Exception {

		//Get the node
		JSONObject recNode = Record.getNode("@[th_"+lang+"_"+key+"]");
		
		//Generate an error if the link already exist
		if (recNode==null) {
			
			throw new Exception("Sorry, the link @[th_"+lang+"_"+key+"] does not exist.");
			
		}
		
		return recNode.get("v")+"";

	}
	
	//Get a link
	public static String get_relation(String lang, String key) throws Exception {

		//Get the node
		JSONObject recNode = Record.getNode("@[r_"+lang+"_"+key+"]");
		
		//Generate an error if the link already exist
		if (recNode==null) {
			
			throw new Exception("Sorry, the link @[r_"+lang+"_"+key+"] does not exist.");
			
		}
		
		return recNode.get("v")+"";

	}
	
	//Check if a link already exist
	public static boolean exist_thought(String lang, String key) throws Exception {

		//Get the node
		JSONObject recNode = Record.getNode("@[th_"+lang+"_"+key+"]");
		
		if (recNode==null) {
			
			return false;
			
		} else {
			
			return true;
			
		}

	}
	
	//Check if a link already exist
	public static boolean exist_relation(String lang, String key) throws Exception {

		//Get the node
		JSONObject recNode = Record.getNode("@[r_"+lang+"_"+key+"]");
		
		if (recNode==null) {
			
			return false;
			
		} else {
			
			return true;
			
		}

	}

}
