/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.rest;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.net.ssl.HttpsURLConnection;

public class REST {
	
	//HTTP request
	public static String http_exe(String method, String url, String urlAction) throws Exception {
		
		//Initialization
		String result = "";

		//Check if the url is null or empty
		if (url==null || url.equals("")) {

			//Generate an exception
			throw new Exception("Sorry, the url cannot be empty.");

		}

		//Generate an error if the method is not valid
		if (method==null || method.equals("")) {

			//Generate an exception
			throw new Exception("Sorry the method cannot be empty.");

		}

		//Generate an error if the method is not valid
		if (!method.equals("GET") && !method.equals("POST") && !method.equals("DELETE") && !method.equals("PUT") && !method.equals("PATCH")) {

			//Generate an exception
			throw new Exception("Sorry the method must be GET|POST|PUT|DELETE|PATCH.");

		}

		try {

			//Create the url and open the connection
			URL postURL = new URL( (method.equals("POST") || method.equals("PUT") || method.equals("PATCH")?url:url+"?"+urlAction) );
			HttpURLConnection conn = (HttpURLConnection)postURL.openConnection();

			// Set connection parameters. We need to perform input and output,
			// so set both as true.
			conn.setDoInput (true);
			conn.setDoOutput (true);

			// Set the content type we are POSTing.
			// encoded form data
			conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
			conn.setRequestProperty( "User-Agent", "MentDB/JPayet.re" );

			//Set the method
			conn.setRequestMethod(method);
			
			//Manage POST method
			if (method.equals("POST") || method.equals("PUT") || method.equals("PATCH")) {
				
				conn.setRequestProperty("Content-length", String.valueOf(urlAction.length())); 
				
				DataOutputStream output = new DataOutputStream(conn.getOutputStream());  
				output.writeBytes(urlAction);
				output.close();
				
			}

			// Read input from the input stream.
			int rc = conn.getResponseCode();
			if ( rc != -1) {

				//Initialization
				BufferedReader is = new BufferedReader(new InputStreamReader( conn.getInputStream()));
				String _line = null;

				//Parse the stream
				while(((_line = is.readLine()) !=null))
				{

					//Get the current line
					result += _line;

				}

			}

		} catch( Exception e ) {

			//An error was generated
			throw new Exception("Sorry an exception appears: "+e.getMessage());

		}

		//Return the html page
		return result;

	}
	
	
	public static String https_exe(String method, String url, String urlAction) throws Exception {
		
		//Initialization
		String result = "";

		//Check if the url is null or empty
		if (url==null || url.equals("")) {

			//Generate an exception
			throw new Exception("Sorry, the url cannot be empty.");

		}

		//Generate an error if the method is not valid
		if (method==null || method.equals("")) {

			//Generate an exception
			throw new Exception("Sorry the method cannot be empty.");

		}

		//Generate an error if the method is not valid
		if (!method.equals("GET") && !method.equals("POST") && !method.equals("DELETE") && !method.equals("PUT") && !method.equals("PATCH")) {

			//Generate an exception
			throw new Exception("Sorry the method must be GET|POST|PUT|DELETE|PATCH.");

		}

		try {

			//Create the url and open the connection
			URL postURL = new URL( (method.equals("POST") || method.equals("PUT") || method.equals("PATCH")?url:url+"?"+urlAction) );
			HttpsURLConnection conn = (HttpsURLConnection)postURL.openConnection();

			// Set connection parameters. We need to perform input and output,
			// so set both as true.
			conn.setDoInput (true);
			conn.setDoOutput (true);

			// Set the content type we are POSTing.
			// encoded form data
			conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
			conn.setRequestProperty( "User-Agent", "MentDB/JPayet.re" );

			//Set the method
			conn.setRequestMethod(method);
			
			//Manage POST method
			if (method.equals("POST") || method.equals("PUT") || method.equals("PATCH")) {
				
				conn.setRequestProperty("Content-length", String.valueOf(urlAction.length())); 
				
				DataOutputStream output = new DataOutputStream(conn.getOutputStream());  
				output.writeBytes(urlAction);
				output.close();
				
			}

			// Read input from the input stream.
			int rc = conn.getResponseCode();
			if ( rc != -1) {

				//Initialization
				BufferedReader is = new BufferedReader(new InputStreamReader( conn.getInputStream()));
				String _line = null;

				//Parse the stream
				while(((_line = is.readLine()) !=null))
				{

					//Get the current line
					result += _line;

				}

			}

		} catch( Exception e ) {

			//An error was generated
			throw new Exception("Sorry an exception appears: "+e.getMessage());

		}

		//Return the html page
		return result;

	}

}
