/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.user;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.basic.BasicData;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.entity.language.LanguageManager;
import re.jpayet.mentdb.core.job.JobManager;
import re.jpayet.mentdb.core.parameter.ParameterManager;
import re.jpayet.mentdb.core.script.ScriptManager;
import re.jpayet.mentdb.core.sequence.SequenceManager;

//Grant management
public class GroupManager {
	
	//Create the group object
	public static void init(String user) throws Exception {
		
		JSONObject seq = new JSONObject();
		
		Record.add(user, "G[]", seq.toJSONString());
		
	}
	
	//Check if a group already exist
	public static boolean exist(String groupName) throws IOException {
		
		if (Record.getNode("G["+groupName+"]")==null) {
			return false;
		} else {
			return true;
		}
		
	}
	
	//Show all groups
	public static String showAllGroups() throws Exception {
		
		return Record.getNode("G[]").toJSONString();
		
	}
	
	//Get group
	@SuppressWarnings("unchecked")
	public static JSONArray userGetGroup(String login) throws Exception {
		
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		JSONArray result = new JSONArray();
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (((JSONArray) bdGroup.dataNode.get("users")).contains(login)) {
				
				result.add(k);
				
			}
			
		}
		
		return result;
		
	}
	
	//Get group
	@SuppressWarnings("unchecked")
	public static JSONArray scriptGetGroup(String scriptName) throws Exception {
		
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		JSONArray result = new JSONArray();
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (((JSONArray) bdGroup.dataNode.get("scripts")).contains(scriptName)) {
				
				result.add(k);
				
			}
			
		}
		
		return result;
		
	}
	
	//Get script
	@SuppressWarnings("unchecked")
	public static String userGetScript(String login) throws Exception {
		
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		JSONArray result = new JSONArray();
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (((JSONArray) bdGroup.dataNode.get("users")).contains(login)) {
				
				JSONArray scritps = ((JSONArray) bdGroup.dataNode.get("scripts"));
				
				for(int i=0;i<scritps.size();i++) {
					
					String currentScript = scritps.get(i)+"";
					
					if (!result.contains(currentScript)) {
						
						result.add(currentScript);
					
					}
					
				}
				
			}
			
		}
		
		return result.toJSONString();
		
	}
	
	//Get user
	@SuppressWarnings("unchecked")
	public static String scriptGetUser(String scriptName) throws Exception {
		
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		JSONArray result = new JSONArray();
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (((JSONArray) bdGroup.dataNode.get("scripts")).contains(scriptName)) {
				
				JSONArray users = ((JSONArray) bdGroup.dataNode.get("users"));
				
				for(int i=0;i<users.size();i++) {
					
					String currentUser = users.get(i)+"";
					
					if (!result.contains(currentUser)) {
						
						result.add(currentUser);
					
					}
					
				}
				
			}
			
		}
		
		return result.toJSONString();
		
	}
	
	//Group to HTML
	@SuppressWarnings("unchecked")
	public static String toHtml(String login) throws Exception {
				
		//Initialization
		JSONObject json = new JSONObject();
		JSONObject core = new JSONObject();
		json.put("core", core);
		JSONArray data = new JSONArray();
		core.put("data", data);
		
		JSONObject mentalScript = new JSONObject();
		mentalScript.put("text", "Mental script");
		mentalScript.put("icon", "images"+File.separator+"mental.png");
		JSONObject opened = new JSONObject();
		opened.put("opened", true);
		mentalScript.put("state", opened);
		data.add(mentalScript);
		
		mentalScript.put("title1", "Add");
		mentalScript.put("action1", "script create \"scriptName\" 0 \"[v1],[v2]\" {\n	\n	+ [v1] [v2];\n	\n};");
		mentalScript.put("direct1", "0");
		
		JSONArray children = null;
		
		children = new JSONArray();
		mentalScript.put("children", children);
		
		//Load the node
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (!k.equals("all") && (((JSONArray) bdGroup.dataNode.get("users")).contains(login) || login.equals("admin"))) {
				
				JSONArray scripts = (JSONArray) bdGroup.dataNode.get("scripts");
				
				if (scripts.size()>0) {
				
					JSONObject group = new JSONObject();
					group.put("text", k);
					group.put("icon", "images/folder.png");
					children.add(group);
					JSONArray groupChildren = new JSONArray();
					group.put("children", groupChildren);
					
					for(int i=0;i<scripts.size();i++) {
					
						JSONObject mental = new JSONObject();
						mental.put("text", ""+scripts.get(i));
						mental.put("icon", "images/mental.png");
						groupChildren.add(mental);
						JSONArray mentalChildren = new JSONArray();
						mental.put("children", mentalChildren);
						mental.put("mql", "in editor {\n	script generate execute \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";\n};");
						mental.put("direct", "1");

						mental.put("title1", "Edit");
						mental.put("action1", "in editor {\n	script generate update \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";\n};");
						mental.put("direct1", "1");
						
						mental.put("title2", "Include");
						mental.put("action2", "in editor {\n	script generate include \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";\n};");
						mental.put("direct2", "1");
						
						mental.put("title3", "Show user");
						mental.put("action3", "script show users \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";");
						mental.put("direct3", "1");
						
						mental.put("title4", "Show group");
						mental.put("action4", "script show groups \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";");
						mental.put("direct4", "1");
						
						mental.put("title5", "Grant group");
						mental.put("action5", "group grant script \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\" \"group\";");
						mental.put("direct5", "0");
						
						mental.put("title6", "Ungrant group");
						mental.put("action6", "group ungrant script \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\" \"group\";");
						mental.put("direct6", "0");
						
						mental.put("title7", "REST Access");
						mental.put("action7", ScriptManager.generateUrl(""+scripts.get(i), login));
						mental.put("direct7", "0");
						
						mental.put("title8", "Delete");
						mental.put("action8", "script delete \""+(scripts.get(i)+"").replace("\"", "\\\"")+"\";");
						mental.put("direct8", "0");
						
					}
					
				}
				
			}
			
		}
		
  		JSONObject addUser = new JSONObject();
  		addUser.put("text", "User");
  		addUser.put("icon", "images/user1.png");
		data.add(addUser);
		JSONArray userChildren = new JSONArray();
		addUser.put("children", userChildren);
		
		addUser.put("title1", "Add");
		addUser.put("action1", "user create \"login\" \"password\";");
		addUser.put("direct1", "0");
		
		addUser.put("title2", "DB Sessions");
		addUser.put("action2", "sessions;");
		addUser.put("direct2", "1");
		
		addUser.put("title3", "Current DB session id");
		addUser.put("action3", "sid;");
		addUser.put("direct3", "1");
		
		addUser.put("title4", "WS Sessions");
		addUser.put("action4", "@sessions;");
		addUser.put("direct4", "1");
		
		addUser.put("title5", "Current WS session id");
		addUser.put("action5", "@sid;");
		addUser.put("direct5", "1");
  		
  		JSONArray userList = UserManager.showAll("mentdb");
  		
  		for(int i=0;i<userList.size();i++) {
  			
  			String u = userList.get(i)+"";
  			
  			if (!u.equals("ai") && !u.equals("mentdb")) {
  			
	  			JSONObject user = new JSONObject();
	  			
	  			user.put("text", ""+u);
	  			user.put("icon", "images/user1.png");
	  			userChildren.add(user);
				JSONArray userChildrenAction = new JSONArray();
				user.put("children", userChildrenAction);
				user.put("mql", "node show U["+u+"];");
				user.put("direct", "1");
				
				user.put("title1", "Show node");
				user.put("action1", "node show U["+u+"];");
				user.put("direct1", "1");
				
				user.put("title2", "Show thought");
				user.put("action2", "node show {\n	node select element U["+u+"] \"/th\"; \n};");
				user.put("direct2", "1");
				
				user.put("title3", "Show user groups");
				user.put("action3", "user show groups \""+(u+"").replace("\"", "\\\"")+"\";");
				user.put("direct3", "1");
				
				user.put("title4", "Show user scripts");
				user.put("action4", "user show scripts \""+(u+"").replace("\"", "\\\"")+"\";");
				user.put("direct4", "1");
				
				user.put("title5", "Grant group");
				user.put("action5", "group grant user \""+(u+"").replace("\"", "\\\"")+"\" \"group\";");
				user.put("direct5", "0");
				
				user.put("title6", "Ungrant group");
				user.put("action6", "group ungrant user \""+(u+"").replace("\"", "\\\"")+"\" \"group\";");
				user.put("direct6", "0");
				
				user.put("title7", "Update password");
				user.put("action7", "user set password \""+(u+"").replace("\"", "\\\"")+"\" \"password\";");
				user.put("direct7", "0");
				
				user.put("title8", "Get token");
				user.put("action8", "user token \""+(u+"").replace("\"", "\\\"")+"\";");
				user.put("direct8", "1");
				
				user.put("title9", "Reset token");
				user.put("action9", "user reset token \""+(u+"").replace("\"", "\\\"")+"\";");
				user.put("direct9", "0");
				
				user.put("title10", "Disable (Cannot be deleted)");
				user.put("action10", "user disable \""+(u+"").replace("\"", "\\\"")+"\";");
				user.put("direct10", "0");
				
  			}
  			
  		}
  		
  		JSONObject addGroup = new JSONObject();
  		addGroup.put("text", "Group");
  		addGroup.put("icon", "images/folder.png");
		data.add(addGroup);
		JSONArray groupChildren = new JSONArray();
		addGroup.put("children", groupChildren);
		
		addGroup.put("title1", "Add");
		addGroup.put("action1", "group add \"groupName\";");
		addGroup.put("direct1", "0");
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			if (!k.equals("all")) {
			
				JSONObject group = new JSONObject();
				group.put("text", k);
				group.put("icon", "images/folder.png");
				groupChildren.add(group);
				JSONArray groupChildrenAction = new JSONArray();
				group.put("children", groupChildrenAction);
				group.put("mql", "group get \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct", "1");
				
				group.put("title1", "Show");
				group.put("action1", "group get \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct1", "1");
				
				group.put("title2", "Show user");
				group.put("action2", "group get user \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct2", "1");
				
				group.put("title3", "Show script");
				group.put("action3", "group get script \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct3", "1");
				
				group.put("title4", "Grant user");
				group.put("action4", "group grant user \"login\" \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct4", "0");
				
				group.put("title5", "Grant script");
				group.put("action5", "group grant script \"scriptName\" \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct5", "0");
				
				group.put("title6", "Ungrant user");
				group.put("action6", "group ungrant user \"login\" \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct6", "0");
				
				group.put("title7", "Ungrant script");
				group.put("action7", "group ungrant script \"scriptName\" \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct7", "0");
				
				group.put("title8", "Delete");
				group.put("action8", "group remove \""+k.replace("\"", "\\\"")+"\";");
				group.put("direct8", "0");
				
			}
			
		}
		
  		JSONObject job = new JSONObject();
  		job.put("text", "Job");
  		job.put("icon", "images/timer.png");
		data.add(job);
		JSONArray jobChildrenAction = new JSONArray();
		job.put("children", jobChildrenAction);
		
		job.put("title1", "Add");
		job.put("action1", "job add \"jobId\" \"scriptName\" \"0 30 * * * ?\";");
		job.put("direct1", "0");
		
		JSONObject jobs = JobManager.show();
		
		for(Object o: jobs.keySet()) {
			
			String currentJob = (String) o;
			
			JSONObject curJob = new JSONObject();
			curJob.put("text", currentJob);
			curJob.put("icon", "images/timer.png");
			jobChildrenAction.add(curJob);
			curJob.put("mql", "in editor {\n	job generate update \""+currentJob.replace("\"", "\\\"")+"\";\n};");
			curJob.put("direct", "1");
			
			curJob.put("title1", "Edit");
			curJob.put("action1", "in editor {\n	job generate update \""+currentJob.replace("\"", "\\\"")+"\";\n};");
			curJob.put("direct1", "1");
			
			curJob.put("title2", "Delete");
			curJob.put("action2", "job remove \""+currentJob.replace("\"", "\\\"")+"\";");
			curJob.put("direct2", "0");
			
		}
		
		job.put("title2", "Status");
		job.put("action2", "job scheduler status;");
		job.put("direct2", "1");
		
		job.put("title3", "Start");
		job.put("action3", "job scheduler start;");
		job.put("direct3", "0");
		
		job.put("title4", "Stop");
		job.put("action4", "job scheduler stop;");
		job.put("direct4", "0");
		
		job.put("title5", "Restart");
		job.put("action5", "job scheduler restart;");
		job.put("direct5", "0");
		
  		JSONObject addSequence = new JSONObject();
  		addSequence.put("text", "Sequence");
  		addSequence.put("icon", "images/seq.png");
		data.add(addSequence);
		JSONArray sequenceChildren = new JSONArray();
		addSequence.put("children", sequenceChildren);
		
		addSequence.put("title1", "Add");
		addSequence.put("action1", "sequence add \"key\" \"value\";");
		addSequence.put("direct1", "0");
		
		bd = new BasicData();  bd.load(SequenceManager.showAllSeqs());
		JSONObject seqs = bd.dataNode;
		
		for(Object o: seqs.keySet()) {
			
			String currentSeq = (String) o;
			
			JSONObject curSeq = new JSONObject();
			curSeq.put("text", currentSeq);
			curSeq.put("icon", "images/seq.png");
			sequenceChildren.add(curSeq);
			curSeq.put("mql", "in editor {\n	sequence generate update \""+currentSeq.replace("\"", "\\\"")+"\";\n};");
			curSeq.put("direct", "1");
			
			curSeq.put("title1", "Edit");
			curSeq.put("action1", "in editor {\n	sequence generate update \""+currentSeq.replace("\"", "\\\"")+"\";\n};");
			curSeq.put("direct1", "1");
			
			curSeq.put("title2", "Delete");
			curSeq.put("action2", "sequence remove \""+currentSeq.replace("\"", "\\\"")+"\";");
			curSeq.put("direct2", "0");
			
		}
		
  		JSONObject lang = new JSONObject();
  		lang.put("text", "Language");
  		lang.put("icon", "images/flag.png");
		data.add(lang);
		JSONArray languageChildren = new JSONArray();
		lang.put("children", languageChildren);
  		
  		lang.put("title1", "Add");
  		lang.put("action1", "language create \"langId\";");
  		lang.put("direct1", "0");
  		
  		JSONArray language = LanguageManager.showAll();
  		for(int i=0;i<language.size();i++) {
			
			String currentLang = (String) language.get(i);
			
			JSONObject curLang = new JSONObject();
			curLang.put("text", currentLang);
			curLang.put("icon", "images/flag.png");
			languageChildren.add(curLang);
			curLang.put("mql", "node show L["+currentLang+"]");
			curLang.put("direct", "1");
			
		}
		
  		JSONObject addParam = new JSONObject();
  		addParam.put("text", "Parameter");
  		addParam.put("icon", "images/conf.png");
		data.add(addParam);
		JSONArray paramChildrenAction = new JSONArray();
		addParam.put("children", paramChildrenAction);
  		
		addParam.put("title1", "Add");
		addParam.put("action1", "parameter add \"param\" \"value\" 0;");
		addParam.put("direct1", "0");
		
		JSONObject params = ParameterManager.showAllParams("mentdb");
		
		for(Object o: params.keySet()) {
			
			String currentParam = (String) o;
			
			JSONObject curParam = new JSONObject();
			curParam.put("text", currentParam);
			curParam.put("icon", "images/conf.png");
			paramChildrenAction.add(curParam);
			curParam.put("mql", "in editor {\n	parameter generate update \""+currentParam.replace("\"", "\\\"")+"\";\n};");
			curParam.put("direct", "1");
			
			curParam.put("title1", "Edit");
			curParam.put("action1", "in editor {\n	parameter generate update \""+currentParam.replace("\"", "\\\"")+"\";\n};");
			curParam.put("direct1", "1");
			
			curParam.put("title2", "Delete");
			curParam.put("action2", "parameter remove \""+currentParam.replace("\"", "\\\"")+"\";");
			curParam.put("direct2", "0");
			
		}
		
  		JSONObject brain = new JSONObject();
  		brain.put("text", "Brain");
  		brain.put("icon", "images/brain.png");
		data.add(brain);
		JSONArray brainChildrenAction = new JSONArray();
		brain.put("children", brainChildrenAction);
		brain.put("mql", "brain status;");
		brain.put("direct", "1");
		
		brain.put("title1", "Status");
		brain.put("action1", "brain status;");
		brain.put("direct1", "1");
		
		brain.put("title2", "Start");
		brain.put("action2", "brain start;");
		brain.put("direct2", "0");
		
		brain.put("title3", "Stop");
		brain.put("action3", "brain stop;");
		brain.put("direct3", "0");
		
  		JSONObject fs = new JSONObject();
  		fs.put("text", "File System");
  		fs.put("icon", "images/fs.png");
		data.add(fs);
		JSONArray fsChildrenAction = new JSONArray();
		fs.put("children", fsChildrenAction);
		fs.put("mql", "metric index files 1 10;");
		fs.put("direct", "1");
		
		fs.put("title1", "Index file");
		fs.put("action1", "metric index files 1 10;");
		fs.put("direct1", "1");
		
		fs.put("title2", "Data file");
		fs.put("action2", "metric data files 1 10;");
		fs.put("direct2", "1");
		
		fs.put("title3", "Transaction logs");
		fs.put("action3", "transaction logs 10;");
		fs.put("direct3", "1");
		
		fs.put("title4", "Show log file");
		fs.put("action4", "log show;");
		fs.put("direct4", "1");
		
		fs.put("title5", "Boot config file");
		fs.put("action5", "config server;");
		fs.put("direct5", "1");
		
  		JSONObject help = new JSONObject();
  		help.put("text", "Help");
  		help.put("icon", "images/help.png");
  		help.put("mql", "help;");
  		help.put("direct", "1");
		data.add(help);
		
	  	return "j23i88m90m76i39t04r09y35p14a96y09e57t35"+json.toJSONString();
		
	}
	
	//Show a group
	public static String get(String user, String groupName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		}
		
		return Record.getNode("G["+groupName+"]").toJSONString();
		
	}
	
	//Get user
	public static String getUser(String user, String groupName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		}
		
		JSONObject bd = Record.getNode("G["+groupName+"]");
		
		return ((JSONArray) bd.get("users")).toJSONString();
		
	}
	
	//Get script
	public static String getScript(String user, String groupName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		}
		
		JSONObject bd = Record.getNode("G["+groupName+"]");
		
		return ((JSONArray) bd.get("scripts")).toJSONString();
		
	}
	
	//Get group for script
	@SuppressWarnings("unchecked")
	public static JSONArray getGroupForScript(String user, String scriptName) throws Exception {
		
		if (scriptName.length()==0) {
			
			throw new Exception("Sorry, the script name is required.");
			
		}
		
		if (!ScriptManager.exist(scriptName)) {
			
			throw new Exception("Sorry, the script <"+scriptName+"> does not exist.");
			
		}
		
		BasicData bd = new BasicData();  bd.load(showAllGroups());
		JSONArray result = new JSONArray();
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( bd.dataNode.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o1.getKey() ).compareTo( o2.getKey() );
			}
		} );
		
		for (Map.Entry<String, Integer> entry : list) {
			String k = entry.getKey();
			
			//Get the group
			BasicData bdGroup = new BasicData();  bdGroup.load(GroupManager.get("mentdb", k));
			
			if (((JSONArray) bdGroup.dataNode.get("scripts")).contains(scriptName)) {
				
				if (!result.contains(k)) {
					
					result.add(k);
				
				}
				
			}
			
		}
		
		return result;
		
	}
	
	//Add a group
	@SuppressWarnings("unchecked")
	public static void add(String user, String groupName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> already exist.");
			
		} else {
			
			//Get the node
			JSONObject recNode = Record.getNode("G[]");
			
			recNode.put(groupName, "G["+groupName+"]");
			
			Record.update(user, "G[]", recNode.toJSONString());
		
			//Build the value
			JSONObject group = new JSONObject();
			group.put("k", groupName);
			group.put("users", new JSONArray());
			group.put("scripts", new JSONArray());
			
			//Add the record
			Record.add(user, "G["+groupName+"]", group.toJSONString());
		
		}
		
	}
	
	//Delete a group
	public static void delete(String user, String groupName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else {
			
			if (groupName.equals("all") || groupName.equals("system") || groupName.equals("library")
					 || groupName.equals("private") || groupName.equals("demo") || groupName.equals("api-mql")
					 || groupName.equals("api-ai") || groupName.equals("api-rest")) {
				
				throw new Exception("Sorry, the group <"+groupName+"> is a system group.");
				
			}
			
			//Get the node
			JSONObject recNode = Record.getNode("G[]");
			
			recNode.remove(groupName);
			
			Record.update(user, "G[]", recNode.toJSONString());
		
			//Add the record
			Record.remove(user, "G["+groupName+"]");
		
		}
		
	}
	
	//Grant user
	@SuppressWarnings("unchecked")
	public static void grantUser(String user, String groupName, String login) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (login.length()==0) {
			
			throw new Exception("Sorry, the login is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!UserManager.exist(login)) {
			
			throw new Exception("Sorry, the user <"+login+"> does not exist.");
			
		} else {
			
			if (isGrantedUser(user, groupName, login)) {
				
				throw new Exception("Sorry, the user is already granted.");
				
			}
			
			if (!user.equals("mentdb") && !GroupManager.isGrantedUser(user, "system", user)) {
				
				throw new Exception("Sorry, only a user in 'system' group can add user right."); 
				
			}
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			((JSONArray) recNode.get("users")).add(login);
			
			Record.update(user, "G["+groupName+"]", recNode.toJSONString());
		
		}
		
	}
	
	//Ungrant user
	public static void ungrantUser(String user, String groupName, String login) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (login.length()==0) {
			
			throw new Exception("Sorry, the login is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!UserManager.exist(login)) {
			
			throw new Exception("Sorry, the user <"+login+"> does not exist.");
			
		} else {
			
			if (!isGrantedUser(user, groupName, login)) {
				
				throw new Exception("Sorry, the user is not granted.");
				
			}
			
			if (!user.equals("mentdb") && !GroupManager.isGrantedUser(user, "system", user)) {
				
				throw new Exception("Sorry, only a user in 'system' group can remove user right."); 
				
			}
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			((JSONArray) recNode.get("users")).remove(login);
			
			Record.update(user, "G["+groupName+"]", recNode.toJSONString());
		
		}
		
	}
	
	//Check if a user is granted
	public static boolean isGrantedUser(String user, String groupName, String login) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (login.length()==0) {
			
			throw new Exception("Sorry, the login is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!UserManager.exist(login)) {
			
			throw new Exception("Sorry, the user <"+login+"> does not exist.");
			
		} else {
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			if (((JSONArray) recNode.get("users")).contains(login)) return true;
			else return false;
		
		}
		
	}
	
	//Grant script
	@SuppressWarnings("unchecked")
	public static void grantScript(String user, String groupName, String scriptName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (scriptName.length()==0) {
			
			throw new Exception("Sorry, the script name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!ScriptManager.exist(scriptName)) {
			
			throw new Exception("Sorry, the script <"+scriptName+"> does not exist.");
			
		} else {
			
			if (isGrantedScript(user, groupName, scriptName)) {
				
				throw new Exception("Sorry, the script is already granted.");
				
			}
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			((JSONArray) recNode.get("scripts")).add(scriptName);
			
			Record.update(user, "G["+groupName+"]", recNode.toJSONString());
		
		}
		
	}
	
	//Ungrant script
	public static void ungrantScript(String user, String groupName, String scriptName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (scriptName.length()==0) {
			
			throw new Exception("Sorry, the script name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!ScriptManager.exist(scriptName)) {
			
			throw new Exception("Sorry, the script <"+scriptName+"> does not exist.");
			
		} else {
			
			if (!isGrantedScript(user, groupName, scriptName)) {
				
				throw new Exception("Sorry, the script is not granted.");
				
			}
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			((JSONArray) recNode.get("scripts")).remove(scriptName);
			
			Record.update(user, "G["+groupName+"]", recNode.toJSONString());
		
		}
		
	}
	
	//Check if a script is granted
	public static boolean isGrantedScript(String user, String groupName, String scriptName) throws Exception {
		
		if (groupName.length()==0) {
			
			throw new Exception("Sorry, the group name is required.");
			
		}
		
		if (scriptName.length()==0) {
			
			throw new Exception("Sorry, the script name is required.");
			
		}
		
		if (!exist(groupName)) {
			
			throw new Exception("Sorry, the group <"+groupName+"> does not exist.");
			
		} else if (!ScriptManager.exist(scriptName)) {
			
			throw new Exception("Sorry, the script <"+scriptName+"> does not exist.");
			
		} else {
			
			//Get the node
			JSONObject recNode = Record.getNode("G["+groupName+"]");
			
			if (((JSONArray) recNode.get("scripts")).contains(scriptName)) return true;
			else return false;
			
		}
		
	}
	
}
